from flask import Blueprint, request
from flask_restful import Api, Resource

from portal.service import service
from portal.tools.decorators import error_handler
from portal.tools.logging import log
from portal.database.models import Group
from portal.rest import group_schema, groups_schema, users_schema
from portal.service.errors import PortalAPIError

groups = Blueprint('groups', __name__, url_prefix='/courses/<string:cid>/groups')
groups_api = Api(groups)


class GroupResource(Resource):
    @error_handler
    def get(self, cid, gid):
        log.info(f"GET to {request.url}")
        course = service.find_course(cid)
        group = service.find_group(course, gid)
        return group_schema.dump(group)

    @error_handler
    def delete(self, cid, gid):
        log.info(f"DELETE to {request.url}")
        course = service.find_course(cid)
        group = service.find_group(course, gid)
        service.delete_entity(group)
        return '', 204

    @error_handler
    def put(self, cid, gid):
        log.info(f"PUT to {request.url}")
        course = service.find_course(cid)
        group = service.find_group(course, gid)
        json_data = request.get_json()
        if not json_data:
            raise PortalAPIError(400, message='No data provided for group update.')

        data = group_schema.load(json_data)[0]

        group.name = data['name']
        service.write_entity(group)
        log.debug(f"Updated group {gid} to {group}.")
        return '', 204


class GroupsList(Resource):
    @error_handler
    def get(self, cid):
        log.info(f"GET to {request.url}")
        course = service.find_course(cid)

        return groups_schema.dump(course.groups)

    @error_handler
    def post(self, cid):
        log.info(f"POST to {request.url}")
        course = service.find_course(cid)
        json_data = request.get_json()
        if not json_data:
            raise PortalAPIError(400, message='No data provided for group creation.')

        data = group_schema.load(json_data)[0]

        new_group = Group(course=course, name=data['name'])
        service.write_entity(new_group)
        log.info(f"Created group={new_group} for course {cid}")
        return group_schema.dump(new_group)[0], 201


class GroupUsersList(Resource):
    @error_handler
    def get(self, cid, gid):
        log.info(f"GET to {request.url}")

        course = service.find_course(cid)
        group = service.find_group(course, gid)

        return users_schema.dump(group.users)

    @error_handler
    def put(self, cid, gid):
        log.info(f"PUT to {request.url}")

        course = service.find_course(cid)
        group = service.find_group(course, gid)

        json_data = request.get_json()
        if not json_data:
            raise PortalAPIError(400, message='No data provided for group user list update.')

        data = users_schema.load(json_data)[0]

        # everything from users_add is added, THEN everything from users_remove is subtracted
        users_old = set(group.users)
        users_add = data.get('add')
        users_remove = data.get('remove')
        if users_add is not None:
            users_old.union(users_add)
        if users_remove is not None:
            users_old.difference(users_remove)

        log.info(f"Updated users of group {gid} to {data}.")
        return '', 204


class GroupUser(Resource):
    @error_handler
    def put(self, cid, gid, uid):
        log.info(f"PUT to {request.url}")
        course = service.find_course(cid)
        group = service.find_group(course, gid)
        user = service.find_user(uid)

        if user not in group.users:
            group.users.append(user)
            log.info(f"Added user {uid} to group {gid} in course {cid}.")
        else:
            log.info(f"User {uid} is already in group {gid} in course {cid}: no change.")
        return '', 204

    @error_handler
    def delete(self, cid, gid, uid):
        log.info(f"DELETE to {request.url}")
        course = service.find_course(cid)
        group = service.find_group(course, gid)
        user = service.find_user(uid)

        try:
            group.users.remove(user)
        except ValueError:
            raise PortalAPIError(400, message=f"Could not remove user {uid} from group {gid} in course {cid}: "
                                              f"group does not contain user.")
        log.info(f"Removed user {uid} from group {gid} in course {cid}.")
        return '', 204


groups_api.add_resource(GroupResource, '/<string:gid>')
groups_api.add_resource(GroupsList, '/')
groups_api.add_resource(GroupUsersList, '/<string:gid>/users')
groups_api.add_resource(GroupUser, '/<string:gid>/users/<string:uid>')
