from flask import Blueprint, request
from flask_restful import Api, Resource

from portal.rest import user_schema, users_schema, submissions_schema, roles_schema, courses_schema, groups_schema, \
    reviews_schema
from portal.service.errors import PortalAPIError, ValidationError
from portal.service import service
from portal.database.models import User
from portal.tools.logging import log
from portal.tools.decorators import error_handler

users = Blueprint('users', __name__, url_prefix='/users')
users_api = Api(users)


class UserResource(Resource):
    @error_handler
    def get(self, uid):
        log.info(f"GET to {request.url}")
        user = service.find_user(uid)
        return user_schema.dump(user)[0], 200

    @error_handler
    def put(self, uid):
        log.info(f"PUT to {request.url}")
        user = service.find_user(uid)

        json_data = request.get_json()
        if not json_data:
            raise PortalAPIError(400, message='No data provided for user update.')
        data = user_schema.load(json_data)
        if data[1]:
            raise ValidationError(schema=str(user_schema), data=json_data, failures=data[1])
        data = data[0]

        user.name = data['name']
        user.email = data['email']
        user.uco = data['uco']
        user.username = data['username']
        user.is_admin = data['is_admin']
        # TODO all; care for password
        service.write_entity(user)
        log.debug(f"Updated user {uid} to {user}.")
        return '', 204

    @error_handler
    def delete(self, uid):
        log.info(f"DELETE to {request.url}")
        user = service.find_user(uid)
        service.delete_entity(user)
        return '', 204


class UserList(Resource):
    @error_handler
    def get(self):
        log.info(f"GET to {request.url}")
        course_id = request.args.get('course')
        group_id = request.args.get('group')
        if group_id and not course_id:
            raise PortalAPIError(400, "Invalid filter combination at /get/users: missing course id.")

        users = service.find_users_filtered(course_id=course_id, group_id=group_id)
        return users_schema.dump(users)[0], 200

    @error_handler
    def post(self):
        log.info(f"POST to {request.url}")
        json_data = request.get_json()
        if not json_data:
            raise PortalAPIError(400, message='No data provided for user creation.')
        data = user_schema.load(json_data)
        if data[1]:
            raise ValidationError(schema=str(user_schema), data=json_data, failures=data[1])
        data = data[0]

        admin = data.get('is_admin') or False
        new_user = User(uco=data['uco'], email=data['email'], username=data['username'], is_admin=admin)
        service.write_entity(new_user)
        log.debug(f"Created user={User}")
        return user_schema.dump(new_user)[0], 201


class UserSubmissionList(Resource):
    # needs more tests
    @error_handler
    def get(self, uid):
        log.info(f"GET to {request.url}")
        course_id = request.args.get('course')
        project_id = request.args.get('project')
        user = service.find_user(uid)

        if project_id and not course_id:
            raise PortalAPIError(400, f"Invalid filter combination at /get/users/{uid}/submissions: missing course id.")

        submissions = service.find_submissions_filtered(user=user, course_id=course_id, project_id=project_id)
        return submissions_schema.dump(submissions)[0], 200


class UserRoleList(Resource):
    @error_handler
    def get(self, uid):
        log.info(f"GET to {request.url}")
        user = service.find_user(uid)
        course_id = request.args.get('course')

        roles = service.find_roles_filtered(user=user, course_id=course_id)
        return roles_schema.dump(roles)[0], 200


class UserCourseList(Resource):
    @error_handler
    def get(self, uid):
        log.info(f"GET to {request.url}")
        user = service.find_user(uid)
        return courses_schema.dump(user.courses)[0], 200


class UserGroupList(Resource):
    @error_handler
    def get(self, uid):
        log.info(f"GET to {request.url}")
        user = service.find_user(uid)
        course_id = request.args.get('course')

        groups = service.find_groups_filtered(user=user, course_id=course_id)
        return groups_schema.dump(groups)[0], 200


class UserReviewList(Resource):
    @error_handler
    def get(self, uid):
        log.info(f"GET to {request.url}")
        user = service.find_user(uid)

        reviews = service.find_reviews(user)
        return reviews_schema.dump(reviews)[0], 200


users_api.add_resource(UserResource, '/<string:uid>')
users_api.add_resource(UserList, '/')
users_api.add_resource(UserSubmissionList, '/<string:uid>/submissions')
users_api.add_resource(UserRoleList, '/<string:uid>/roles')
users_api.add_resource(UserCourseList, '/<string:uid>/courses')
users_api.add_resource(UserGroupList, '/<string:uid>/groups')
users_api.add_resource(UserReviewList, '/<string:uid>/reviews')
