from portal.service.errors import ResourceNotFoundError
from portal.tools.logging import log
from portal import db
from portal.database.models import Course, Role, User, Project, Group, Submission, SubmissionState
from portal import storage


def delete_entity(entity):
    db.session.delete(entity)
    db.session.commit()


def write_entity(entity):
    db.session.add(entity)
    db.session.commit()


def find_submissions_filtered(user: User, course_id: str, project_id: str):
    submissions = Submission.query.filter_by(user=user)  # user.submissions cannot be used as the base query
    if course_id:
        submissions = submissions.join(Project)
        if project_id:
            submissions = submissions.filter((Project.id == project_id) | (Project.name == project_id))
        submissions = submissions.join(Course).filter((Course.id == course_id) | (Course.codename == course_id))

    return submissions


def find_users_filtered(course_id: str, group_id: str):
    users = User.query
    if course_id:
        users = users.join(User.roles) \
            .join(Course).filter((Course.id == course_id) | (Course.codename == course_id))
    if group_id:
        users = users.join(User.groups).filter((Group.id == group_id) | (Group.name == group_id))

    return users


def find_groups_filtered(user: User, course_id: str):
    # not using db query to filter yet... user does not have that many groups
    groups = user.groups
    if course_id:
        groups = [group for group in groups if group.course.id == course_id or group.course.codename == course_id]

    return groups


def find_roles_filtered(user: User, course_id: str):
    # not using db query to filter yet... user does not have that many roles
    roles = user.roles
    if course_id:
        roles = [role for role in roles if role.course.id == course_id or role.course.codename == course_id]
    return roles


def find_reviews(user: User):
    reviews = []
    for item in user.review_items:
        if item.review not in reviews:
            reviews.append(item.review)
    return reviews


def find_resource(identifier: str, resource: str, query):
    """Gets an instance of the resource
    Args:
        identifier(str): Id or name or some other identifier of the resource
        resource(str): Resource name
        query: SQLAlchemy query

    Returns: A entity instance

    """
    log.debug(f"Find {resource} by identifier: {identifier}")
    instance = query.first()

    if not instance:
        log.debug(f"Not found {resource} for identifier: {identifier}")
        raise ResourceNotFoundError(resource=resource, res_id=identifier)

    log.debug(f"Found {resource}: {instance}")
    return instance


def find_submission(identifier: str) -> Submission:
    return find_resource(
        resource='submission',
        identifier=identifier,
        query=Submission.query.filter_by(id=identifier)
    )


def find_course(identifier: str) -> Course:
    query = Course.query.filter(
        ((Course.id == identifier) | (Course.codename == identifier))
    )

    return find_resource(
        resource='course',
        identifier=identifier,
        query=query
    )


def find_project(course, identifier: str) -> Project:
    query = Project.query.filter_by(course=course).filter(
        (Project.id == identifier) | (Project.name == identifier)
    )

    return find_resource(
        resource='project',
        identifier=identifier,
        query=query
    )


def find_group(course: Course, identifier: str) -> Group:
    query = Group.query.filter_by(course=course).filter(
        (Group.id == identifier) | (Group.name == identifier)
    )

    return find_resource(
        resource='group',
        identifier=identifier,
        query=query
    )


def find_role(course: Course, identifier: str) -> Role:
    query = Role.query.filter_by(course=course).filter(
        (Role.id == identifier) | (Role.name == identifier)
    )

    return find_resource(
        resource='role',
        identifier=identifier,
        query=query
    )


def find_user(identifier: str) -> User:
    query = User.query.filter(
        ((User.id == identifier) | (User.username == identifier) | (User.email == identifier))
    )

    return find_resource(
        resource='user',
        identifier=identifier,
        query=query
    )


def can_create_submission(user: User, project: Project) -> bool:
    """Checks if a user may create a new submission.

    A user is permitted to create a new submission if there are no submissions
    created or queued for him in the given project.

    Args:
        user(User): the user who wants to crete a new submission
        project(Project): the project in which the user wants to create a submission

    Returns: whether the user can create a new submission in the project

    """
    conflict = Submission.query.filter(Submission.user == user) \
        .filter(Submission.project == project) \
        .filter(Submission.state.in_(
        [SubmissionState.CREATED, SubmissionState.READY, SubmissionState.QUEUED])).first()

    return conflict is None


def create_submission(user, project, file_params, project_params_string) -> Submission:
    """Creates a new submission in the database and downloads its files.

    Zip and git sources are processed differently.

    Args:
        user(User): the submission's author
        project(Project): the project the submission falls under
        file_params(dict): parameters for Storage - how and where to download files
        project_params_string(str): custom parameters specified by project with their values as a JSON-encoded string

    Returns(Submission): the created submission entity

    """
    # adding review at submission create is not supported
    # TODO: store all configuration relevant for submission scheduling:
    # project.submissions_allowed from, to, archive; project.config.submission_scheduler_config, submission_params?
    new_submission = Submission(user=user, project=project, parameters=project_params_string)
    write_entity(new_submission)

    # download relevant files into Storage
    file_params['whitelist'] = project.config.file_whitelist
    if file_params['source']['type'] == 'zip':
        file_params['source']['path'] = "todo"  # TODO: find out how to download files through flask
    storage.submissions.create(new_submission.id, **file_params)
    return new_submission
