from portal.database.models import Course
from . import utils
import json


def test_list(client):
    response = client.get('/courses/')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = str(response.get_data().decode("utf-8"))
    courses = json.loads(data)
    assert len(courses) == 2
    db_courses = Course.query.all()
    utils.assert_course_in(db_courses, courses[0])


def test_create(client):
    request_dict = dict(
        name="java",
        codename="PA165",
    )
    db_courses_number = len(Course.query.all())

    request_json = json.dumps(request_dict)
    response = client.post('/courses/', data=request_json, headers={"content-type": "application/json"})
    assert response.status_code == 201
    assert response.mimetype == 'application/json'

    resp_new_course = json.loads(str(response.get_data().decode("utf-8")))
    db_courses = Course.query.all()
    assert len(db_courses) == db_courses_number + 1
    utils.assert_course_in(db_courses, resp_new_course)
    print(db_courses)


def test_create_extra_key(client):
    request_dict = dict(
        name="java",
        codename="PA165",
        extra="should not be here"
    )
    db_courses_number = len(Course.query.all())

    request_json = json.dumps(request_dict)
    response = client.post('/courses/', data=request_json, headers={"content-type": "application/json"})
    assert response.status_code == 201
    assert response.mimetype == 'application/json'

    resp_new_course = json.loads(str(response.get_data().decode("utf-8")))
    db_courses = Course.query.all()
    assert len(db_courses) == db_courses_number + 1
    utils.assert_course_in(db_courses, resp_new_course)


def test_create_missing_key(client):
    request_dict = dict(
        codename="PA165",
    )
    db_courses_number = len(Course.query.all())

    request_json = json.dumps(request_dict)
    response = client.post('/courses/', data=request_json, headers={"content-type": "application/json"})
    assert response.status_code == 400
    assert response.mimetype == 'application/json'
    assert len(Course.query.all()) == db_courses_number


def test_create_invalid_value(client):
    request_dict = dict(
        name=456,
        codename="PA165",
    )
    db_courses_number = len(Course.query.all())

    request_json = json.dumps(request_dict)
    response = client.post('/courses/', data=request_json, headers={"content-type": "application/json"})
    assert response.status_code == 400
    assert response.mimetype == 'application/json'
    assert len(Course.query.all()) == db_courses_number


def test_read(client):
    c = Course.query.filter_by(codename="PB161").first()
    response = client.get(f"/courses/{c.id}")
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = str(response.get_data().decode("utf-8"))
    course = json.loads(data)

    utils.assert_course(c, course)


def test_update(client):
    c = Course.query.filter_by(codename="PB161").first()
    request_dict = dict(
        name="new_cpp",
        codename="PB161_new",
    )
    request_json = json.dumps(request_dict)
    response = client.put(f'/courses/{c.id}', data=request_json, headers={"content-type": "application/json"})

    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    assert not Course.query.filter_by(codename="PB161").first()
    course_from_db = Course.query.filter_by(codename="PB161_new").first()
    assert course_from_db.name == "new_cpp"


def test_delete(client):
    c = Course.query.filter_by(codename="PB161").first()
    response = client.delete(f'/courses/{c.id}', headers={"content-type": "application/json"})

    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    assert not Course.query.filter_by(codename="PB161").first()


def test_notes_token_read(client):
    c = Course.query.filter_by(codename="PB161").first()
    response = client.get(f"/courses/{c.id}/notes_access_token")
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = json.loads(str(response.get_data().decode("utf-8")))
    assert data == "token"


def test_notes_token_create(client):
    c = Course.query.filter_by(codename="PB071").first()
    response = client.put(f"/courses/{c.id}/notes_access_token", data=json.dumps("new_token"),
                          headers={"content-type": "application/json"})
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    updated = Course.query.filter_by(codename="PB071").first()
    assert updated.notes_access_token == "new_token"


def test_notes_token_update(client):
    c = Course.query.filter_by(codename="PB161").first()
    response = client.put(f"/courses/{c.id}/notes_access_token", data=json.dumps("new_token"),
                          headers={"content-type": "application/json"})
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    updated = Course.query.filter_by(codename="PB161").first()
    assert updated.notes_access_token == "new_token"
